@@ -3,7 +3,7 @@ import os
 from dataclasses import dataclass
 from json import JSONDecodeError
 
-from .ynam_parser import arg
+from .ynam_parser import arg, logger
 
 
 @dataclass
@@ -39,11 +39,17 @@ def loadSecrets():
 
 
 def updateStash(key, value):
-    if value != '':
-        secrets = {**loadSecrets()}
-        secrets[key] = value
-        with open(arg('config_file'), 'w+') as file:
-            file.write(json.dumps(secrets, indent=2))
+    secrets = {**loadSecrets()}
+    if value == '':
+        logger.info(
+            f'Empty value for key: {key}. Maintaining current value: {secrets[key]}')  # noqa
+        return
+
+    logger.debug(f'Updating key: {key} to value: {value}')
+    secrets[key] = value
+    with open(arg('config_file'), 'w+') as file:
+        logger.debug('Persisting to disk...')
+        file.write(json.dumps(secrets, indent=2))
 
 
 stash = Secrets(**loadSecrets())
